#ifndef __AUDIOH__
#define __AUDIOH__

#include <stdio.h>

#ifdef WIN32 
#  include <windows.h>
#  include <mmsystem.h>
#  include <mmreg.h>
#  include <dsound.h>
#  define strncasecmp strnicmp
#  define strcasecmp  stricmp
#  define EXPORT(a,b) __declspec(dllexport) a b
#else
#  define min(a,b) ((a)<(b)?(a):(b))
#  define max(a,b) ((a)>(b)?(a):(b))
#  define EXPORT(a,b) a b
#endif

#ifdef Solaris
#  include <sys/types.h>
#  include <sys/file.h>
#  include <sys/ioctl.h>
#  include <sys/fcntl.h>
#  include <stropts.h>
#  include <sys/errno.h>
#  include <sys/audioio.h>
#  include <errno.h>
#  include <sys/filio.h>
#endif

#define IDLE    0
#define READ    1
#define WRITE   2
#define PAUSED  3
  
#ifndef WIN32
#endif
  
#define MAXNBLKS 200
#define SEXP 18
#define CEXP (SEXP+1)
#define SBLKSIZE (1<<SEXP)
#define CBLKSIZE (1<<CEXP)

#define SSAMPLE(buf, i) buf[((i)&(SBLKSIZE-1))]
#define UCSAMPLE(buf, i) ((unsigned char *)buf)[((i)&(CBLKSIZE-1))]
#define CSAMPLE(buf, i) ((char *)buf)[((i)&(CBLKSIZE-1))]

#define HEADBUF 1024
#define QUE_STRING "QUE"
#define RAW_STRING "RAW"
#define WAV_STRING "WAV"
#define AIFF_STRING "AIFF"
#define AU_STRING "AU"


#define NMAX 4096
#define NMIN 8

#define SND_FORMAT_MULAW_8   1
#define SND_FORMAT_LINEAR_8  2
#define SND_FORMAT_LINEAR_16 3
#define SND_FORMAT_ALAW_8    27

#define GUESS_FFT_LENGTH 512

#define DEFAULT_MULAW_FREQ 8000
#define DEFAULT_ALAW_FREQ 8000
#define DEFAULT_LIN8OFFSET_FREQ 11025
#define DEFAULT_LIN8_FREQ 11025

  
#define ITEMBUFFERSIZE 100000
  
  typedef enum {
    GUESS_LIN16,
    GUESS_LIN16S,
    GUESS_ALAW,
    GUESS_MULAW,
    GUESS_LIN8OFFSET,
    GUESS_LIN8
  } sampleEncoding;


  typedef enum {
    NATIVE,
    BIGENDIAN,
    LITTLEENDIAN
  } Endianness;

  enum {
    NIST_HEADERSIZE = 1024,
    AU_HEADERSIZE = 28,
    WAV_HEADERSIZE = 44,
    AIFF_HEADERSIZE = 54
  };

namespace Audio {
#define VOLBUFSIZE 20
#define JACKBUFSIZE 40
#define QUERYBUFSIZE 1000
#define MAX_DEVICE_NAME_LENGTH 100
  
  extern int strncasecmp(const char *s1, const char *s2, size_t n);

#define BUFSIZE 512

#define RECORD 1
#define PLAY   2
  
  enum {
		LIN16      = 1,
		ALAW       = 2,
    MULAW      = 3,
    LIN8OFFSET = 4,
    LIN8       = 5
  };
  
#define CAPABLEN 100

  enum {
    MONO   = 1,
    STEREO = 2,
    QUAD   = 4,
  };
}

#endif
