#ifndef __AUDIOFILE__
#define __AUDIOFILE__

#include "Audio.h"

namespace Audio {
  
  class AFfilehandle {  
  public:
    int    sampfreq;
    int    sampformat;
    int    sampsize;
    int    nchannels;
    int    length;
    int    maxlength;
    short  maxsamp;
    short  minsamp;
    int    abmax;
    short  **blocks;
    int    maxblks;
    int    nblks;
    int    nPlayed;
    int    startPos;
    int    filePos;
    int    totLen;
    short  *tmpbuf;
    unsigned char *buffer;
    int    swap;
    int    storeType;
    int    headSize;
    int    skipBytes;
    int    buffersize;
    int    active;
    int    inByteOrder;  
    int firstNRead;

    AFfilehandle(int _sampfreq, int _sampformat, int _nchannels);
    ~AFfilehandle();
  };

  class File {
  private:  
    FILE
      *fp;  
    AFfilehandle 
      *filehandle;
    char
      *filename;
    char
      *filetype;


  public:
    File(int _sampfreq, int _sampformat, int _nchannels);
    ~File();

    int Open(char *_filename, char *mode);
    int Close();
    int Seek(int pos);


    char *GuessWav(char *buf, int len);
    char *GuessAu(char *buf, int len);
    char *GuessAiff(char *buf, int len);
    int GuessFileType(char *buf, int len, int eof);    
    short GetSample(int index);

    void SetLength(int len) {
      filehandle->length = len;
    }
    int GetLength() {
      return filehandle->length;
    }
    void SetFrequency(int freq) {
      filehandle->sampfreq = freq;
    }
    int GetFrequency() {
      return filehandle->sampfreq;
    }
    void SetSampleFormat(int fmt) {
      filehandle->sampformat = fmt;
    }
    int GetSampleFormat() {
      return filehandle->sampformat;
    }
    void SetNumChannels(int nchan) {
      filehandle->nchannels = nchan;
    }
    int GetNumChannels() {
      return filehandle->nchannels;
    }
    void SetBytesPerSample(int n) {
      filehandle->sampsize = n;
    }
    int GetBytesPerSample() {
      return filehandle->sampsize;
    }
    void SetHeaderSize(int size) {
      filehandle->headSize = (size);
    }
    char *GetFilename() {
      return filename;
    }
    void GetExtremes(int start, int end,int *pmax, int *pmin);
    void UpdateExtremes(int start, int end, int flag);
    short SwapShort(short s);
    long SwapLong(long l);
    void ByteSwapSound();
    void SwapIfBE();
    void SwapIfLE();
  
    int ExtCmp(char *s1, char *s2);
    char *ExtWavFile(char *s);
    char *ExtAuFile(char *s);
    char *ExtAiffFile(char *s);
    int GuessFormat(unsigned char *buf, int len);
    int GetRawHeader(char *buf);
    int PutRawHeader(int len);
    int GetAuHeader(char *buf);
    int PutAuHeader(int len);
    int GetHeaderBytes(char *buf, int len);
    int GetWavHeader(char *buf);
    int PutWavHeader(int len);
    unsigned long ConvertFloat(unsigned char *buffer);
    void StoreFloat(unsigned char * buffer, unsigned long value);
    int GetAiffHeader(char *buf);
    int PutAiffHeader(int len);

    short ReadBEShort();
    short ReadLEShort();
    long ReadBELong();
    long ReadLELong();
#ifdef WIN32
    int Write(char *buf, int len);
#else
    int Write(short *buf, int len);
#endif
    int WriteLEShort(short s);
    int WriteLELong(long l);
    int WriteBEShort(short s);
    int WriteBELong(long l);
    long GetLELong(char *buf, int pos);
    short GetLEShort(char *buf, int pos);
    long GetBELong(char *buf, int pos);
    short GetBEShort(char *buf, int pos);
    void PutBELong(char *buf, int pos, long l);
    void PutBEShort(char *buf, int pos, short s);
    void PutLELong(char *buf, int pos, long l);
    void PutLEShort(char *buf, int pos, short s);
  };
  
}

#endif /* _SOUND */
