#ifndef __AUDIOPORT__
#define __AUDIOPORT__

#include "Audio.h"


#ifdef Solaris
#  include <sys/types.h>
#  include <sys/file.h>
#  include <sys/ioctl.h>
#  include <sys/fcntl.h>
#  include <stropts.h>
#  include <sys/errno.h>
#  include <sys/audioio.h>
#  include <errno.h>
#  include <sys/filio.h>
#endif

#ifdef IRIX
#  include <audio.h>
#endif


#ifdef WIN
#  include <windows.h>
#  include <mmsystem.h>
#  include <mmreg.h>
#  include <dsound.h>
#endif

#ifdef Linux
#include <sys/time.h>

namespace Time {

  double Current();
}

namespace String {

  char *Duplicate(const char *str);
}
#endif

namespace Audio {
  
#ifdef IRIX
#  include <audio.h>
#endif
  
#ifdef Linux
#  define DEVICE_NAME "/dev/dsp"
#endif

  typedef struct ADesc {

#ifdef OSS
    int    
      afd,
      // count,
      frag_size,
      freq,
      convert,
      warm;
    double 
      time,
      timep;
#endif

#ifdef Solaris
    audio_info_t 
      ainfo;
    int 
      afd,
      freq,
      convert,
      convSize;
    double 
      time,
      timep;
    short 
      *convBuf;
#endif

#ifdef WIN32
    int 
      curr,
      freq,
      shortRead,
      convert,
      BufLen,
      written;
    unsigned int 
      BufPos;
    PCMWAVEFORMAT 
      pcmwf,
      pcmwfPB;
    // PLAY
#  ifdef _DSOUND
    DSBUFFERDESC 
      dsbdesc;
    LPDIRECTSOUNDBUFFER 
      lplpDsb;
    LPDIRECTSOUNDBUFFER 
      lplpDsPB;
    DSBUFFERDESC 
      dsbdescPB;
    // RECORD
    DSCBUFFERDESC 
      dscbdesc;
    LPDIRECTSOUNDCAPTUREBUFFER 
      lplpDscb;
#  endif
#endif

#ifdef Linux
    int    afd;
    int    count;
    int    frag_size;
    double time;
    double timep;
    int    freq;
    int    convert;
#endif

#ifdef IRIX
  ALport   
    port;
  ALconfig 
    config;
  unsigned long long 
    startfn;
  int 
    count;
#endif

  int 
    bytesPerSample,
    nChannels,
    mode,
    debug;

} ADesc;

// Multimedia-Library
#define NBUFS 64
#define QUERYBUFSIZE 1000
#define MAX_DEVICE_NAME_LENGTH 100

// DirectSound
#define NSECS 1
#define NUMCAPTUREEVENTS 26
#define MAX_NUM_DEVICES 20

  class Port {
  private:
    int 
      bufSize;
   ADesc
     desc;
    
#ifdef WIN32
    bool 
      stop;
    
    // Multimedia-Library 
    int				
      devIndex;
    int 
      correction;
    // Play
    HWAVEOUT
      hWaveOut;
    WAVEFORMATEX 
      wFormatOut;
    WAVEHDR
      waveHdrOut[NBUFS];
    HANDLE 
      blockOut[NBUFS];
    int           
      blockSizeOut[NBUFS];
    char 
      *outDeviceList[MAX_NUM_DEVICES];
    int
      numOutDevs;
    // Record
    HWAVEIN  
      hWaveIn;
    WAVEHDR
      waveHdrIn[NBUFS];
    char 
      *blockIn[NBUFS];
    WAVEINCAPS
      wInCaps;
    WAVEOUTCAPS 
      wOutCaps;
    char 
      *inDeviceList[MAX_NUM_DEVICES];
    int 
      numInDevs;
    
    // Direct Sound
    HRESULT 
      hr; 
   FILE 
     *fp; 
   
#  ifdef _DSOUND
   HINSTANCE 
     hDSoundInstance;
   LPDIRECTSOUND 
     lpDirectSound;
   LPDIRECTSOUNDCAPTURE 
     lpDSCapture;
   HANDLE
     rghEvent[NUMCAPTUREEVENTS];
   DSBPOSITIONNOTIFY 
     rgdscbpn[NUMCAPTUREEVENTS];
   LPDIRECTSOUNDNOTIFY 
     lpdsNotify;
#  endif
#else 
  char
    *defaultDeviceName;
#endif

  public:
#ifdef WIN32
#  ifdef _DSOUND
   // Direct Sound
   // Play
   char 
     *DSOutDeviceList[MAX_NUM_DEVICES];
   int
     numDSOutDevs;
   GUID 
     guidOut[4];
   
   // Record
   char 
     *DSInDeviceList[MAX_NUM_DEVICES];
   int 
     numDSInDevs;
   GUID
     guidIn[4];
#  endif
#endif

    Port();
    ~Port();

    void Init();
#ifdef WIN32
    int  Open(int mode, int freq, int channels, int encoding);
#else
    int  Open(int mode, int freq, int channels, int encoding, char *device=NULL);
#endif
    int  Close();
    int  Pause();
    void Resume();
    void Flush();
    void Stop();
    void Post();
    int  Read(void *buf, int nSamples);
    int  Write(void *buf, int nSamples);
    int  Readable();
    int  Played();
    int  Writeable();

#ifdef WIN32
    int  GetDevices();
		void GetFormats(char *device, char *buf, int); 
#else
    int  GetDevices(char **arr, int n);
    void GetFormats(char *buf, int n);
#endif

    void GetFrequencies(char *buf, int n);
    int  GetMixerDevices(char **arr, int n);
    int  GetBufSize() {return bufSize;}
  };
}

#endif /* _AUDIO */
