#define INITGUID 
#include <objbase.h> 
#include <cguid.h>  

#include "AudioPort.h"
#include "g711.h"



Audio::Port::Port()
{
	int
		i;

  stop = false;
  //Direct Sound
	numInDevs    = 0;
	numOutDevs   = 0;
#ifdef _DSOUND
	numDSOutDevs = 0;
	numDSInDevs  = 0;
#endif

	for (i=0; i<MAX_NUM_DEVICES; i++) {
		outDeviceList[i]   = NULL;
		inDeviceList[i]    = NULL;
#ifdef _DSOUND
		DSOutDeviceList[i] = NULL;
		DSInDeviceList[i]  = NULL;
#endif
	}

	correction  = 1;

#ifdef _DSOUND
  lpDSCapture = NULL;
#endif

  // Multimedia Library

  for (i = 0; i < NBUFS; i++) 
	  blockIn[i] = NULL;
}

Audio::Port::~Port()
{
	int
		i;

	for (i=0; i<MAX_NUM_DEVICES; i++) {
		if (outDeviceList[i]   != NULL) free (outDeviceList[i]);
		if (inDeviceList[i]    != NULL) free (inDeviceList[i]);
#ifdef _DSOUND
		if (DSOutDeviceList[i] != NULL) free (DSOutDeviceList[i]);
		if (DSInDeviceList[i]  != NULL) free (DSInDeviceList[i]);
#endif
	}
}

#ifdef _DSOUND
static BOOL CALLBACK DSEnumProc(LPGUID lpGUID, LPSTR lpszDesc, LPSTR lpszDrvName, LPVOID lpContext)
{
  Audio::Port 
    *port;

  port = (Audio::Port *)lpContext;

  if (port->numDSOutDevs == MAX_NUM_DEVICES) return(TRUE);

  if (lpGUID != NULL) {
    memcpy(&(port->guidOut[port->numDSOutDevs]), lpGUID, sizeof(GUID));
  } 

  port->DSOutDeviceList[port->numDSOutDevs] = (char *)malloc(strlen(lpszDesc)+15);
  if (port->DSOutDeviceList[port->numDSOutDevs] != NULL) {
    strcpy(port->DSOutDeviceList[port->numDSOutDevs], lpszDesc);
    strcat(port->DSOutDeviceList[port->numDSOutDevs], " (DirectSound)");
  }
  port->numDSOutDevs++;
  
  return(TRUE);
}

static BOOL CALLBACK DSCEnumProc(LPGUID lpGUID, LPSTR lpszDesc, LPSTR lpszDrvName, LPVOID lpContext)
{
  Audio::Port 
    *port;

  port = (Audio::Port *)lpContext;

  if (port->numDSInDevs == MAX_NUM_DEVICES) return(TRUE);
  if (lpGUID != NULL) {
    memcpy(&(port->guidIn[port->numDSInDevs]), lpGUID, sizeof(GUID));
  } 

  port->DSInDeviceList[port->numDSInDevs] = (char *)malloc(strlen(lpszDesc)+15);
  if (port->DSInDeviceList[port->numDSInDevs] != NULL) {
    strcpy(port->DSInDeviceList[port->numDSInDevs], lpszDesc);
    strcat(port->DSInDeviceList[port->numDSInDevs], " (DirectSound)");
  }
  port->numDSInDevs++;
  
  return(TRUE);
}

#endif

void Audio::Port::Init()
{
/*  int 
    i;

  if (!(numInDevs == numOutDevs == numDSInDevs == numDSOutDevs == 0)) return;
  if (useDSound) {
    hDSoundInstance = NULL; 
    hDSoundInstance = LoadLibrary("dsound.dll");
    if (hDSoundInstance == NULL) {
      useDSound = false;
    }
    else {
      hr = DirectSoundCreate(NULL, &lpDirectSound, NULL);
      if (hr == DS_OK) {
        hr = DirectSoundEnumerate((LPDSENUMCALLBACK) DSEnumProc, (LPVOID) this);
        if (hr != DS_OK) {
	        MessageBox(NULL, "failed DirectSoundEnumerate", NULL,MB_OK);
	        return; 
        }
        hr = DirectSoundCaptureEnumerate((LPDSENUMCALLBACK) DSCEnumProc,(LPVOID) this);
        if (hr != DS_OK) {
	        MessageBox(NULL, "failed DirectSoundCaptureEnumerate", NULL,MB_OK);
	        return; 
        }
      }
      return;
    }
  }

  inDeviceList[numInDevs] = (char *)malloc(strlen("Wave Mapper")+1);
  strcpy(inDeviceList[numInDevs], "Wave Mapper");
  numInDevs++;

  for (i = 0; i < (int)waveInGetNumDevs(); i++) {
    if (waveInGetDevCaps(i, &wInCaps, sizeof(WAVEINCAPS)) == 0) {
      inDeviceList[numInDevs] = (char *)malloc(strlen(wInCaps.szPname)+17);
      if (inDeviceList[numInDevs] != NULL) {
	      strcpy(inDeviceList[numInDevs], wInCaps.szPname);
	      strcat(inDeviceList[numInDevs], " (Win multimedia)");
      }
      numInDevs++;
      if (numInDevs == MAX_NUM_DEVICES) break;
    }
  }

  outDeviceList[numOutDevs] = (char *)malloc(strlen("Wave Mapper")+1);
  strcpy(outDeviceList[numOutDevs], "Wave Mapper");
  numOutDevs++;

  for (i = 0; i < (int)waveOutGetNumDevs(); i++) {
    if (waveOutGetDevCaps(i, &wOutCaps, sizeof(WAVEOUTCAPS)) == 0) {
      outDeviceList[numOutDevs] = (char *)malloc(strlen(wOutCaps.szPname)+17);
      if (outDeviceList[numOutDevs] != NULL) {
        strcpy(outDeviceList[numOutDevs], wOutCaps.szPname);
	      strcat(outDeviceList[numOutDevs], " (Win multimedia)");
      }
      numOutDevs++;
      if (numOutDevs == MAX_NUM_DEVICES) break;
    }
  }*/

}

int Audio::Port::Open(int _mode, int _freq, int _channels, int _encoding)
{

  int 
		i,
    res = 0;

  Init();

  switch (_mode) {
		case RECORD:
#ifndef _DSOUND
	      WAVEFORMATEX  
		      wFormatIn;
        int 
          size;

	      desc.mode      = _mode;
        desc.curr      = 0;
        desc.freq      = _freq;
        desc.nChannels = _channels;
        desc.shortRead = 0;
        desc.convert   = 0;

			  memset(&wFormatIn, 0, sizeof(WAVEFORMATEX));
			  switch (_encoding) {
				  case LIN16:
					  desc.bytesPerSample = sizeof(short);
				  	wFormatIn.wFormatTag = WAVE_FORMAT_PCM;
					  break;
				  case ALAW:
					  desc.bytesPerSample = sizeof(char);
					  wFormatIn.wFormatTag = WAVE_FORMAT_ALAW;
					  break;
			    case MULAW:
					  desc.bytesPerSample = sizeof(char);
					  wFormatIn.wFormatTag = WAVE_FORMAT_MULAW;
					  break;
				  case LIN8OFFSET:
					  desc.bytesPerSample = sizeof(char);
					  wFormatIn.wFormatTag = WAVE_FORMAT_PCM;
					  break;
			  }
			  wFormatIn.nChannels       = _channels;
			  wFormatIn.nSamplesPerSec  = _freq;
			  wFormatIn.nAvgBytesPerSec = _freq * desc.bytesPerSample * _channels;
			  wFormatIn.nBlockAlign     = desc.bytesPerSample * _channels;
			  wFormatIn.wBitsPerSample  = desc.bytesPerSample * 8;
			  wFormatIn.cbSize          = 0; 

			  size = desc.bytesPerSample * _freq / 16;  
        bufSize = size;

        res = waveInOpen(&hWaveIn, WAVE_MAPPER, (WAVEFORMATEX *)&wFormatIn, 0, 0L, CALLBACK_NULL);
        if (res != MMSYSERR_NOERROR) {
			  	return -1;
        }
        for (i = 0; i < NBUFS; i++) {
			  	blockIn[i] = (char *)malloc(size);
			  	if (!blockIn[i]) {
			  		return -1;
				  }
				  memset(&waveHdrIn[i], 0, sizeof(WAVEHDR));
				  waveHdrIn[i].lpData         = blockIn[i];
				  waveHdrIn[i].dwBufferLength = size;
				  waveHdrIn[i].dwFlags        = 0L;
				  waveHdrIn[i].dwLoops        = 0L;
				  res = waveInPrepareHeader(hWaveIn, &waveHdrIn[i], sizeof(WAVEHDR));
				  res = waveInAddBuffer(hWaveIn, &waveHdrIn[i], sizeof(WAVEHDR));
        }
        res = waveInStart(hWaveIn); 
#else  // Direct Sound
        DWORD
          pos;

        DSCBCAPS dscbcaps;
        
	      hr = DirectSoundCaptureCreate(NULL,&lpDSCapture,NULL);
        if (hr != DS_OK) {
	        MessageBox(NULL, "Failed creating capture object.", NULL,MB_OK);
	        return -1; 
        }

        desc.mode      = _mode;
        desc.curr      = 0;
        desc.freq      = _freq;
        desc.nChannels = _channels;
        desc.shortRead = 0;
        //fp = fopen("debug.txt","w");
        memset(&(desc.pcmwf), 0, sizeof(PCMWAVEFORMAT));
        switch (_encoding) {
        case LIN16:
	        desc.bytesPerSample = sizeof(short);
	        desc.pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
	        break;
        case LIN8OFFSET:
          desc.bytesPerSample = sizeof(char);
	        desc.pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
	        break;
        }
        desc.pcmwf.wf.nChannels       = _channels;
        desc.pcmwf.wf.nSamplesPerSec  = _freq;
        desc.pcmwf.wf.nAvgBytesPerSec = _freq * desc.bytesPerSample * _channels;
        desc.pcmwf.wf.nBlockAlign     = desc.bytesPerSample * _channels;
        desc.pcmwf.wBitsPerSample     = desc.bytesPerSample * 8;

        memset(&(desc.dscbdesc), 0, sizeof(DSCBUFFERDESC)); 
        desc.dscbdesc.dwSize = sizeof(DSCBUFFERDESC); 
        desc.dscbdesc.dwFlags = 0; //DSCBCAPS_WAVEMAPPED; 
        desc.dscbdesc.dwBufferBytes = NSECS * desc.pcmwf.wf.nAvgBytesPerSec;
        desc.dscbdesc.dwReserved = 0;
        desc.dscbdesc.lpwfxFormat = (LPWAVEFORMATEX)&(desc.pcmwf);

        desc.lplpDscb = NULL;

        hr = IDirectSoundCapture_CreateCaptureBuffer(lpDSCapture,&(desc.dscbdesc), &(desc.lplpDscb), NULL);
        if (hr != DS_OK) {
          desc.lplpDscb = NULL; 
	        MessageBox(NULL,"Failed creating capture buffer.", NULL,MB_OK);
	        return -1;
        }

        dscbcaps.dwSize = sizeof(DSCBCAPS);
        hr = IDirectSoundCaptureBuffer_GetCaps(desc.lplpDscb,&dscbcaps); 
        if (hr != DS_OK) {
	         MessageBox(NULL, "failed IDirectSoundCaptureBuffer_GetCaps", NULL,MB_OK);
           if (hr == DSERR_INVALIDPARAM) 
              MessageBox (NULL,"DSERR_INVALIDPARAM",NULL,MB_OK);
           if (hr == DSERR_UNSUPPORTED) 
              MessageBox (NULL,"DSERR_UNSUPPORTED",NULL,MB_OK);
           if (hr == DSERR_OUTOFMEMORY) 
              MessageBox (NULL,"DSERR_OUTOFMEMORY",NULL,MB_OK);
           return -1;
        }
        desc.BufLen = dscbcaps.dwBufferBytes;
        bufSize = desc.BufLen;


        rgdscbpn[0].dwOffset = DSBPN_OFFSETSTOP;
        rgdscbpn[0].hEventNotify = rghEvent[0];
        rghEvent[0] = CreateEvent(NULL,FALSE,FALSE,NULL);
        if (rghEvent[0] == NULL) {
	        MessageBox(NULL, "failed rghEvent[0]", NULL,MB_OK);
	        return -1; 
        }
        for (i=1; i<NUMCAPTUREEVENTS;i++) {
          rghEvent[i] = CreateEvent(NULL,FALSE,FALSE,NULL);
          if (rghEvent[i] == NULL) {
	          MessageBox(NULL, "failed rghEvent[i]", NULL,MB_OK);
	          return -1; 
          }
          rgdscbpn[i].dwOffset = (i*bufSize/(NUMCAPTUREEVENTS))-1;
          rgdscbpn[i].hEventNotify = rghEvent[i];
        }


        hr = IDirectSoundCaptureBuffer_QueryInterface(desc.lplpDscb,IID_IDirectSoundNotify,(VOID **)&lpdsNotify);
        if (hr != DS_OK) {
	        MessageBox(NULL,"Failed .", NULL,MB_OK);
	        return -1;
        }

        hr = IDirectSoundNotify_SetNotificationPositions(lpdsNotify,NUMCAPTUREEVENTS,rgdscbpn);
        if (hr != DS_OK) {
	        MessageBox(NULL,"Failed .", NULL,MB_OK);
          IDirectSoundNotify_Release(lpdsNotify);
	        return -1;
        }
  
        hr = IDirectSoundCaptureBuffer_Start(desc.lplpDscb,DSCBSTART_LOOPING);
        if (hr != DS_OK) {
           if (hr == DSERR_INVALIDPARAM) 
              MessageBox (NULL,"DSERR_INVALIDPARAM",NULL,MB_OK);
           if (hr == DSERR_NODRIVER) 
              MessageBox (NULL,"DSERR_NODRIVER",NULL,MB_OK);
           if (hr == DSERR_OUTOFMEMORY) 
              MessageBox (NULL,"DSERR_OUTOFMEMORY",NULL,MB_OK);
           return -1;
        }
        hr = IDirectSoundCaptureBuffer_GetCurrentPosition(desc.lplpDscb,NULL,&pos);
        if (hr != DS_OK) {
	        MessageBox(NULL,"Failed IDirectSoundCaptureBuffer_GetCurrentPosition", NULL,MB_OK);
	        return -1;
        }
        desc.BufPos = pos;
#endif
      break;
    case PLAY:
#ifndef _DSOUND
      memset(&wFormatOut, 0, sizeof(WAVEFORMATEX));
      switch (_encoding) {
				case LIN16:
					desc.bytesPerSample = sizeof(short);
					wFormatOut.wFormatTag = WAVE_FORMAT_PCM;
					break;
				case ALAW:
					desc.bytesPerSample = sizeof(char);
					wFormatOut.wFormatTag = WAVE_FORMAT_ALAW;
					break;
			 case MULAW:
					desc.bytesPerSample = sizeof(char);
					wFormatOut.wFormatTag = WAVE_FORMAT_MULAW;
					break;
			  case LIN8OFFSET:
					desc.bytesPerSample = sizeof(char);
					wFormatOut.wFormatTag = WAVE_FORMAT_PCM;
					break;
      }
      wFormatOut.nChannels       = desc.nChannels;
      wFormatOut.nSamplesPerSec  = desc.freq;
      wFormatOut.nAvgBytesPerSec = desc.freq * desc.bytesPerSample * desc.nChannels;
      wFormatOut.nBlockAlign     = desc.bytesPerSample * desc.nChannels;
      wFormatOut.wBitsPerSample  = desc.bytesPerSample * 8;
      wFormatOut.cbSize          = 0;
      
      if (_encoding == MULAW || _encoding == ALAW) {
				res = waveOutOpen(NULL, devIndex, (WAVEFORMATEX *)&wFormatOut,
			  0, 0L, WAVE_FORMAT_QUERY);
				if (res != MMSYSERR_NOERROR) {
					desc.convert = _encoding;
					desc.bytesPerSample = sizeof(short);
					wFormatOut.wFormatTag = WAVE_FORMAT_PCM;
					wFormatOut.nAvgBytesPerSec = _freq * desc.bytesPerSample * _channels;
				  wFormatOut.nBlockAlign     = desc.bytesPerSample * _channels;
					wFormatOut.wBitsPerSample  = desc.bytesPerSample * 8;
				}
      }
      
      res = waveOutOpen(&hWaveOut, devIndex,
			(WAVEFORMATEX *)&wFormatOut, 0, 0L, CALLBACK_NULL);
      if (res) {
				MessageBox(NULL,"if res","Open",MB_OK);
				return -1;
      }
      for (i = 0; i < NBUFS; i++) {
				blockOut[i] = NULL;
				blockSizeOut[i] = 0;
				waveHdrOut[i].dwFlags = WHDR_DONE;
      }
#endif
      break;
      }

	return 1;

}



int Audio::Port::Close()
{
#ifndef _DSOUND
    int i = 0, res;
#endif

    switch (desc.mode) {
    case RECORD:
#ifdef _DSOUND
        //fclose(fp);
        if (desc.lplpDscb==0) {
	        return(0);
        }
        hr = IDirectSoundCaptureBuffer_Stop(desc.lplpDscb);
        desc.lplpDscb->Release();
        lpDSCapture->Release();
#else
        waveInStop(hWaveIn);
        waveInReset(hWaveIn);
        waveInClose(hWaveIn);
        for (i = 0; i < NBUFS; i++) {
				  waveInUnprepareHeader(hWaveIn, &waveHdrIn[i], sizeof(WAVEHDR));
				  if (blockIn[i]) {
					  free(blockIn[i]);
					  blockIn[i] = NULL;
				  }
        }
#endif
      break;
    case PLAY:
#ifndef _DSOUND
        res = waveOutClose(hWaveOut);
        if (res == WAVERR_STILLPLAYING) return(-1);
        for (i = 0; i < NBUFS; i++) {
			  	waveOutUnprepareHeader(hWaveOut, &waveHdrOut[i], sizeof(WAVEHDR));
				  if (blockOut[i]) {
					  free(blockOut[i]);
					  blockOut[i] = NULL;
				  }
        }
#endif
      break;
    default:
      break;
    }  
 

  return 1;
}

int Audio::Port::Pause()
{ /* Windows multimedia library */

  switch (desc.mode) {
  case RECORD:
#ifdef _DSOUND
      hr = IDirectSoundCaptureBuffer_Stop(desc.lplpDscb);
#endif
    break;
  case PLAY:
#ifdef _DSOUND
      hr = IDirectSoundBuffer_Stop(desc.lplpDsb);
#else
      int res = waveOutPause(hWaveOut);
#endif
    break;
  }

  return(-1);
}

void Audio::Port::Resume()
{ /* Windows multimedia library */ 
  switch (desc.mode) {
  case RECORD:
    break;
  case PLAY:
#ifdef _DSOUND
      hr = desc.lplpDsb->Play(0, 0, DSBPLAY_LOOPING);
#else
      int res = waveOutRestart(hWaveOut);
#endif
    break;
  }

}

void Audio::Port::Flush()
{
	int 
		res = 0;
    
	switch (desc.mode) {
  case RECORD:
//				res = waveInReset(hWaveIn);
    break;    
  case PLAY:
#ifdef _DSOUND
      hr = IDirectSoundBuffer_Stop(desc.lplpDsb);
      desc.written = 0;
#else
      res = waveOutReset(hWaveOut);
#endif
    break;
  }
}

void Audio::Port::Stop()
{
#ifdef _DSOUND
    SetEvent(rghEvent[0]);
#else
    stop = true;
#endif
}


void Audio::Port::Post()
{
}

int Audio::Port::Read(void *buf, int nFrames)
{
#ifdef _DSOUND

    DWORD dwEvt = MsgWaitForMultipleObjects(NUMCAPTUREEVENTS,rghEvent,FALSE,INFINITE,QS_ALLINPUT);
    dwEvt -= WAIT_OBJECT_0;

    if (dwEvt == 0)  {
      //fprintf(fp,"event: %d\n",dwEvt);
      return (-1);
    }

    LPVOID lpvPtr1; 
    DWORD dwBytes1; 
    LPVOID lpvPtr2; 
    DWORD dwBytes2; 
    int size = 0;
    DWORD pos = 0, pos1 = 0;

    hr = IDirectSoundCaptureBuffer_GetCurrentPosition(desc.lplpDscb,&pos1,&pos);

    if (pos == desc.BufPos) {
      return 0;
    }
    else if (pos > desc.BufPos) {
      size = pos - desc.BufPos;
    } 
    else {
      size = desc.BufLen - (desc.BufPos - pos);
    }

    //fprintf(fp,"event: %d, readable: %d , size: %d, nFrame: %d, BufPos: %d, pos: %d, pos1: %d\n",dwEvt,Readable(),size,nFrames,desc.BufPos,pos,pos1);
 /*   if (size > nFrames * (desc.bytesPerSample * desc.nChannels)) {
      size = nFrames * (desc.bytesPerSample * desc.nChannels);
    }*/

    hr = desc.lplpDscb->Lock(desc.BufPos, size, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, NULL);

    if (hr == DS_OK) {
      memcpy(buf, lpvPtr1, dwBytes1);
      if (NULL != lpvPtr2) { 
	      memcpy((char*)buf+dwBytes1, lpvPtr2, dwBytes2); 
      } 
      hr = desc.lplpDscb->Unlock(lpvPtr1, dwBytes1, lpvPtr2,  dwBytes2);

      desc.BufPos = (desc.BufPos + dwBytes1 + dwBytes2) % desc.BufLen; 
      nFrames = size / (desc.bytesPerSample * desc.nChannels);

      if (hr == DS_OK) {
	      return(nFrames);
      }
    }
    return(0); 
#else
    int 
			offset,
			rest,
			res, 
			i, 
			noread, 
			n, nsamps;
    
		res = 0;
    if ((desc.mode == RECORD) && !stop) {
			noread = 1;
			for (i = 0; i < NBUFS; i++) {
				if (waveHdrIn[i].dwFlags & WHDR_DONE) noread = 0;
      }
      if (noread) {
				return(0);
      } 
			n = 0;
      if (desc.shortRead > 0) {
				offset = desc.shortRead * (desc.bytesPerSample * desc.nChannels);
				rest = waveHdrIn[desc.curr].dwBytesRecorded - offset;
	
				memcpy((char *)buf, blockIn[desc.curr] + offset, rest);
				waveInUnprepareHeader(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
				waveHdrIn[desc.curr].lpData = blockIn[desc.curr];
				waveHdrIn[desc.curr].dwBufferLength = desc.bytesPerSample * desc.nChannels * BUFSIZE;
				waveHdrIn[desc.curr].dwFlags = 0L;
				waveHdrIn[desc.curr].dwLoops = 0L;
				res = waveInPrepareHeader(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
				res = waveInAddBuffer(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
				desc.curr = (desc.curr + 1) % NBUFS;
				desc.shortRead = 0;
				n = rest / (desc.bytesPerSample * desc.nChannels);
      } 
      
			while ((waveHdrIn[desc.curr].dwFlags & WHDR_DONE) && (n < nFrames)) {
				if ((unsigned int)(nFrames - n) * (desc.bytesPerSample * desc.nChannels) < waveHdrIn[desc.curr].dwBytesRecorded) {
					nsamps = nFrames - n;
					desc.shortRead = nsamps;
					memcpy(((char *)buf + n * desc.bytesPerSample * desc.nChannels),
						blockIn[desc.curr], nsamps * (desc.bytesPerSample * desc.nChannels));
				} 
				else {
					memcpy(((char *)buf + n * desc.bytesPerSample * desc.nChannels),
						blockIn[desc.curr], waveHdrIn[desc.curr].dwBytesRecorded);
					nsamps = waveHdrIn[desc.curr].dwBytesRecorded / (desc.bytesPerSample * desc.nChannels);
					waveInUnprepareHeader(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
					waveHdrIn[desc.curr].lpData = blockIn[desc.curr];
					waveHdrIn[desc.curr].dwBufferLength = desc.bytesPerSample * desc.nChannels * BUFSIZE;
					waveHdrIn[desc.curr].dwFlags = 0L;
					waveHdrIn[desc.curr].dwLoops = 0L;
					res = waveInPrepareHeader(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
					res = waveInAddBuffer(hWaveIn, &waveHdrIn[desc.curr], sizeof(WAVEHDR));
					
					desc.curr = (desc.curr + 1) % NBUFS;
					desc.shortRead = 0;
				}
				n += nsamps;
			}
      return(n); 
    }
    if (stop) return -1;
#endif

	return(0);
}

int Audio::Port::Write(void *buf, int nFrames)
{ /* Windows multimedia library */
    int res, i, nowrit = 1, size = nFrames * desc.bytesPerSample * desc.nChannels;


    if (nFrames == 0) return(0); 
    for (i = 0; i < NBUFS; i++) {
      if (waveHdrOut[i].dwFlags & WHDR_DONE) nowrit = 0;
    }
    if (nowrit) return(0); 
  
    for (i = 0; i < NBUFS; i++) {
      if (waveHdrOut[i].dwFlags & WHDR_DONE) {
				if (size > blockSizeOut[i]) {
					if (blockSizeOut[i]) {
						free(blockOut[i]);
					}
					blockOut[i] = (char *)malloc(size);
					if (!blockOut[i]) return(0);
					blockSizeOut[i] = size;
				} 
				else {
				}
				waveOutUnprepareHeader(hWaveOut, &waveHdrOut[i], sizeof(WAVEHDR));
	
				if (desc.convert) {
					int j;
					short *s;
					s = (short *)blockOut[i];
				  
					for (j = 0; j < nFrames * desc.nChannels; j++) {
						if (desc.convert == ALAW) {
							*s++ = G711::Alaw2Lin(((unsigned char *)buf)[j]);
						}
						else {
							*s++ = G711::Mulaw2Lin(((unsigned char *)buf)[j]);
						}
					}
				}
				else {
					memcpy(blockOut[i], buf, size);
				}
				waveHdrOut[i].lpData = (char *)blockOut[i];
				waveHdrOut[i].dwBufferLength = size;
				waveHdrOut[i].dwFlags = 0L;
				waveHdrOut[i].dwLoops = 0L;
				res = waveOutPrepareHeader(hWaveOut, &waveHdrOut[i], sizeof(WAVEHDR));
				res	= waveOutWrite(hWaveOut, &waveHdrOut[i], sizeof(WAVEHDR));
				break;
      }
    }
  return(nFrames);
  
}

int Audio::Port::Readable()
{/* Windows multimedia library */
    int n = 0;

    if (desc.mode == RECORD) {
#ifdef _DSOUND
        DWORD pos = 0;
        DWORD status = 0;

        hr = desc.lplpDscb->GetStatus(&status);
        if (!(status && DSCBSTATUS_CAPTURING)) return(0);

        hr = desc.lplpDscb->GetCurrentPosition(NULL,&pos);
  
        if (pos > desc.BufPos) {
          pos = pos - desc.BufPos;
        } else {
          pos = desc.BufLen - (desc.BufPos - pos);
        } 
        return(pos / (desc.bytesPerSample * desc.nChannels));
#else
        for (int i = 0; i < NBUFS; i++) {
				  if (waveHdrIn[i].dwFlags & WHDR_DONE) {
  					n += (waveHdrIn[i].dwBufferLength / (desc.bytesPerSample * desc.nChannels));
				  }
        }
        if (desc.shortRead > 0) {
				  n -= (waveHdrIn[desc.curr].dwBufferLength / (desc.bytesPerSample * desc.nChannels));
				  n += desc.shortRead;
        }
#endif
    }

    return(n);

}

int Audio::Port::Writeable()
{/* Windows multimedia library */
    int res = 0, i;

    for (i = 0; i < NBUFS; i++) {
      if (waveHdrOut[i].dwFlags & WHDR_DONE) res = -1;
    }
    return(res);
}

int Audio::Port::Played()
{/* Windows multimedia library */
    MMTIME mmtime;
  
    mmtime.wType = TIME_SAMPLES;
    waveOutGetPosition(hWaveOut, &mmtime, sizeof(MMTIME));

    if ((wFormatOut.wFormatTag == WAVE_FORMAT_MULAW) ||
				(wFormatOut.wFormatTag == WAVE_FORMAT_ALAW)) {
      return(mmtime.u.sample / correction);
    }
    return(mmtime.u.sample);
}

int Audio::Port::GetDevices()
{
	int
		i;

	numInDevs = 0;
  
  inDeviceList[numInDevs] = (char *)malloc(strlen("Wave Mapper")+1);
  strcpy(inDeviceList[numInDevs], "Wave Mapper");
  numInDevs++;

  for (i = 0; (i < (int)waveInGetNumDevs()) && (numInDevs < MAX_NUM_DEVICES); i++) {
    if (waveInGetDevCaps(i, &wInCaps, sizeof(WAVEINCAPS)) == 0) {
      inDeviceList[numInDevs] = (char *)malloc(strlen(wInCaps.szPname)+17);
      if (inDeviceList[numInDevs] != NULL) {
				strcpy(inDeviceList[numInDevs], wInCaps.szPname);
				strcat(inDeviceList[numInDevs], " (Win multimedia)");
		  }
      numInDevs++;
    }
  }

	numOutDevs = 0;
  outDeviceList[numOutDevs] = (char *)malloc(strlen("Wave Mapper")+1);
  strcpy(outDeviceList[numOutDevs], "Wave Mapper");
  numOutDevs++;

  for (i = 0; (i < (int)waveOutGetNumDevs()) && (numOutDevs < MAX_NUM_DEVICES); i++) {
    if (waveOutGetDevCaps(i, &wOutCaps, sizeof(WAVEOUTCAPS)) == 0) {
      outDeviceList[numOutDevs] = (char *)malloc(strlen(wOutCaps.szPname)+17);
      if (outDeviceList[numOutDevs] != NULL) {
				strcpy(outDeviceList[numOutDevs], wOutCaps.szPname);
				strcat(outDeviceList[numOutDevs], " (Win multimedia)");
      }
      numOutDevs++;
    }
  }



	return 1;
}

void Audio::Port::GetFormats(char *device, char *buf, int n)
{
	 WAVEFORMATEX  
		 wFormatOut;
  int 
		res, 
		pos = 0;

    wFormatOut.wFormatTag = WAVE_FORMAT_PCM;
    wFormatOut.nChannels       = 1;
    wFormatOut.nSamplesPerSec  = 11025;
    wFormatOut.nAvgBytesPerSec = 22050;
    wFormatOut.nBlockAlign     = 2;
    wFormatOut.wBitsPerSample  = 16;
    wFormatOut.cbSize          = 0;
    res = waveOutOpen(NULL, WAVE_MAPPER, (WAVEFORMATEX *)&wFormatOut,
		      0, 0L, WAVE_FORMAT_QUERY);
    if (res == MMSYSERR_NOERROR) {
      pos += sprintf(&buf[pos], "%s", "Lin16");
    }
    
    wFormatOut.wFormatTag = WAVE_FORMAT_PCM;
    wFormatOut.nChannels       = 1;
    wFormatOut.nSamplesPerSec  = 11025;
    wFormatOut.nAvgBytesPerSec = 11025;
    wFormatOut.nBlockAlign     = 1;
    wFormatOut.wBitsPerSample  = 8;
    wFormatOut.cbSize          = 0;
    res = waveOutOpen(NULL, WAVE_MAPPER, (WAVEFORMATEX *)&wFormatOut,
		      0, 0L, WAVE_FORMAT_QUERY);
    if (res == MMSYSERR_NOERROR) {
      pos += sprintf(&buf[pos], "%s", " Lin8offset");
    }
    
    wFormatOut.wFormatTag = WAVE_FORMAT_MULAW;
    wFormatOut.nChannels       = 1;
    wFormatOut.nSamplesPerSec  = 11025;
    wFormatOut.nAvgBytesPerSec = 11025;
    wFormatOut.nBlockAlign     = 1;
    wFormatOut.wBitsPerSample  = 8;
    wFormatOut.cbSize          = 0;
    res = waveOutOpen(NULL, WAVE_MAPPER, (WAVEFORMATEX *)&wFormatOut,
		      0, 0L, WAVE_FORMAT_QUERY);
    if (res == MMSYSERR_NOERROR) {
      pos += sprintf(&buf[pos], "%s", " Mulaw");
    }
    
    wFormatOut.wFormatTag = WAVE_FORMAT_ALAW;
    wFormatOut.nChannels       = 1;
    wFormatOut.nSamplesPerSec  = 11025;
    wFormatOut.nAvgBytesPerSec = 11025;
    wFormatOut.nBlockAlign     = 1;
    wFormatOut.wBitsPerSample  = 8;
    wFormatOut.cbSize          = 0;
    res = waveOutOpen(NULL, WAVE_MAPPER, (WAVEFORMATEX *)&wFormatOut,
		      0, 0L, WAVE_FORMAT_QUERY);
    if (res == MMSYSERR_NOERROR) {
      pos += sprintf(&buf[pos], "%s", " Alaw");
    }
}

void Audio::Port::GetFrequencies(char *buf, int n)
{
  strncpy(buf, "8000 11025 16000 22050 32000 44100 48000", n);
  buf[n-1] = '\0';
}


/*??*/


